/**
 *
 * AiCiA - Android IRC Client
 *
 * Copyright (C)2010 GORRY.
 *
 */

package net.gorry.aicia;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.RemoteException;
import android.text.SpannableStringBuilder;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.Toast;

/**
 *
 * IRCクライアント「AiCiA」
 *
 * @author GORRY
 *
 */

public class ActivityMain extends Activity {
	private static final String TAG = "ActivityMain";
	private static final boolean V = false;

	/** */
	public static final int ACTIVITY_SYSTEMCONFIG = 1;
	/** */
	public static final int ACTIVITY_EXAPPLIST = 2;
	/** */
	public static final int ACTIVITY_EXWEBLIST = 3;
	/** */
	public static final int ACTIVITY_IRCCHANNELCONFIG = 4;

	private static Activity me;
	/** */
	private static IIRCService iIRCService;

	/** */
	public static String mCurrentServerName = null;
	/** */
	public static String mCurrentChannel = null;
	/** */
	public static String mTopic = null;

	/** */
	public static boolean mShutdownServiceOnDestroy = false;

	private final IRCServiceReceiver mReceiver = new IRCServiceReceiver();

	private String mInitOpenServerName = null;

	/** */
	public static Layout layout;
	/** */
	public static DoMain doMain;

	/** */
	public static boolean mDonate;

	private static Toast mToast = null;

	// private CountDownLatch mSignaliIRCServiceSetup;


	/* アプリの一時退避
	 * @see android.app.Activity#onSaveInstanceState(android.os.Bundle)
	 */
	@Override
	protected void onSaveInstanceState(final Bundle outState) {
		addVerboseLog(4, TAG, "onSaveInstanceState()");
		layout.saveInstanceState(outState);
	}

	/*
	 * アプリの一時退避復元
	 * @see android.app.Activity#onRestoreInstanceState(android.os.Bundle)
	 */
	@Override
	protected void onRestoreInstanceState(final Bundle savedInstanceState) {
		addVerboseLog(4, TAG, "onRestoreInstanceState()");
		layout.restoreInstanceState(savedInstanceState);
	}

	/*
	 * 作成
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(final Bundle savedInstanceState) {
		addVerboseLog(4, TAG, "onCreate()");
		super.onCreate(savedInstanceState);

		iIRCService = null;

		final Bundle extras = getIntent().getExtras();
		if (extras != null) {
			final boolean b = (extras.getBoolean("donate"));
			if (b) mDonate = true;
		}

		me = this;
		MyAppInfo.setContext(me);
		MyIcon.setContext(me);
		layout = new Layout(ActivityMain.this);
		doMain = new DoMain(ActivityMain.this);
		SystemConfig.setContext(ActivityMain.this);
		layout.setOrientation(true);
		SystemConfig.loadConfig();
		layout.setRotateMode();

		// getWindow().setSoftInputMode(LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
		layout.baseLayout_Create(true);

		// mSignaliIRCServiceSetup = new CountDownLatch(1);
		// myBindService();
		// myRegisterReceiver();

		setContentView(layout.mBaseLayout);

	}

	/*
	 * 再起動
	 * @see android.app.Activity#onRestart()
	 */
	@Override
	public void onRestart() {
		addVerboseLog(4, TAG, "onRestart()");
		super.onRestart();

	}

	/*
	 * 通知アイコンからの起動でIntentが付いてるとき
	 * @see android.app.Activity#onNewIntent(android.content.Intent)
	 */
	@Override
	public void onNewIntent(final Intent intent) {
		addVerboseLog(4, TAG, "onNewIntent()");
		super.onNewIntent(intent);

		mInitOpenServerName = null;
		final Bundle extras = intent.getExtras();
		if (extras != null) {
			mInitOpenServerName = extras.getString("serverName");
		}
	}

	/*
	 * 開始
	 * @see android.app.Activity#onStart()
	 */
	@Override
	public void onStart() {
		addVerboseLog(4, TAG, "onStart()");
		super.onStart();
	}

	/*
	 * 再開
	 * @see android.app.Activity#onResume()
	 */
	@Override
	public void onResume() {
		addVerboseLog(4, TAG, "onResume()");
		super.onResume();

		final ProgressDialog pd = new ProgressDialog(me);
		final Handler h = new Handler();
		final Runnable[] setupService = new Runnable[1];
		final Runnable dismissProgressDialog = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onResume(): dismissProgressDialog");
				pd.dismiss();
			}
		};
		final Runnable postSetupService = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onResume(): postSetupService");
				doMain.setIIRCService(iIRCService);
				layout.setIIRCService(iIRCService);
				layout.reviveLogs();
				setChannelOnResume();
				h.post(dismissProgressDialog);
			}
		};
		final Runnable waitSetupService = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onResume(): waitSetupService");
				new Thread(new Runnable() {	public void run() {
					int count;
					for (count=0; count<100; count++) {
						if (iIRCService != null) break;
						try {
							Thread.sleep(200);
							// mSignaliIRCServiceSetup.await();
						} catch (final InterruptedException e) {
							//
						}
					}
					if (count >= 100) {
						h.post(setupService[0]);
					} else {
						h.post(postSetupService);
					}
				}} ).start();
			}
		};
		setupService[0] = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onResume(): setupService");
				myBindService();
				myRegisterReceiver();
				h.post(waitSetupService);
			}
		};
		final Runnable showProgressDialog = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onResume(): showProgressDialog");
				pd.setTitle(getString(R.string.activitymain_java_progress_bindservice));
				pd.setIndeterminate(true);
				pd.setCancelable(false);
				pd.show();
				h.post(setupService[0]);
			}
		};
		if (iIRCService != null) {
			try {
				// サービス生存確認
				final String packageName = getPackageName();
				final PackageInfo packageInfo = getPackageManager().getPackageInfo(packageName, PackageManager.GET_META_DATA);
				final String version = packageInfo.versionName;
				final String serviceVersion = iIRCService.getVersionString();
				if (version.equals(serviceVersion)) {
					doMain.setIIRCService(iIRCService);
					layout.setIIRCService(iIRCService);
					final boolean status = setChannelOnResume();
					if (status) {
						return;
					}
				}
				iIRCService = null;
			} catch (final Exception e) {
				iIRCService = null;
			}
		}
		h.post(showProgressDialog);

	}

	/**
	 * サービスの復元
	 */
	private boolean setChannelOnResume() {
		addVerboseLog(4, TAG, "resumeService()");
		layout.restoreInputBox();

		try {
			iIRCService.clearLowMemoryNotification();

			final String serverName = mInitOpenServerName;
			final String channel = iIRCService.getCurrentChannel(serverName);
			if (mInitOpenServerName != null) {
				mInitOpenServerName = null;
				if (channel != null) {
					(me).runOnUiThread(new Runnable(){ public void run() {
						doMain.doChangeChannel(serverName, channel, true);
					}});
				}
			}
		} catch (final RemoteException e) {
			return (false);
		}
		return (true);
	}

	/*
	 * 停止
	 * @see android.app.Activity#onPause()
	 */
	@Override
	public synchronized void onPause() {
		addVerboseLog(4, TAG, "onPause()");
		super.onPause();
		layout.saveInputBox();
	}

	/*
	 * 中止
	 * @see android.app.Activity#onStop()
	 */
	@Override
	public void onStop() {
		addVerboseLog(4, TAG, "onStop()");
		super.onStop();
	}

	//
	/*
	 * 破棄
	 * @see android.app.Activity#onDestroy()
	 */
	@Override
	public void onDestroy() {
		addVerboseLog(4, TAG, "onDestroy()");
		super.onDestroy();
		unregisterReceiver(mReceiver);
		if (mShutdownServiceOnDestroy) {
			try {
				if (iIRCService != null) {
					iIRCService.shutdown();
				}
			}
			catch(final RemoteException e) {
				// エラー
			}
		}
		unbindService(ircServiceConn);
	}

	/*
	 * コンフィギュレーション変更
	 * @see android.app.Activity#onConfigurationChanged(android.content.res.Configuration)
	 */
	@Override
	public synchronized void onConfigurationChanged(final Configuration newConfig) {
		addVerboseLog(4, TAG, "onConfigurationChanged()");
		super.onConfigurationChanged(newConfig);

		final ProgressDialog pd = new ProgressDialog(me);
		final Handler h = new Handler();
		final Runnable dismissProgressDialog = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onConfigurationChanged(): dismissProgressDialog");
				pd.dismiss();
			}
		};

		final Runnable doChangeConfiguration = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onConfigurationChanged(): doChangeConfiguration");
				layout.changeConfiguration(newConfig);
				h.post(dismissProgressDialog);
			}
		};

		final Runnable showProgressDialog = new Runnable() {
			public void run() {
				addVerboseLog(4, TAG, "onConfigurationChanged(): showProgressDialog");
				pd.setTitle(getString(R.string.activitymain_java_progress_changeconfiguration));
				pd.setIndeterminate(true);
				pd.setCancelable(false);
				pd.show();
				h.post(doChangeConfiguration);
			}
		};

		h.post(showProgressDialog);

	}

	/*
	 * メインウィンドウにフォーカスが移るときの処理
	 * 子レイアウトを調整するためのエントリポイントとして使う
	 * @see android.app.Activity#onWindowFocusChanged(boolean)
	 */
	@Override
	public synchronized void onWindowFocusChanged(final boolean b) {
		addVerboseLog(4, TAG, "onWindowFocusChanged()");
		layout.updateBaseLayout();
	}

	/**
	 * IRCサービスへのバインド
	 */
	private void myBindService() {
		addVerboseLog(4, TAG, "myBindService()");
		final Intent i1 = new Intent(this, IRCService.class);
		i1.putExtra("donate", mDonate);
		startService(i1);
		final Intent i2 = new Intent(IIRCService.class.getName());
		i2.putExtra("donate", mDonate);
		bindService(i2, ircServiceConn, BIND_AUTO_CREATE);
	}

	/**
	 * IRCサービスのAPI取得処理
	 */
	private final ServiceConnection ircServiceConn = new ServiceConnection() {
		public void onServiceConnected(final ComponentName name, final IBinder service) {
			addVerboseLog(4, TAG, "ircServiceConn: onServiceConnected()");
			final IIRCService i = IIRCService.Stub.asInterface(service);
			addVerboseLog(4, TAG, "ircServiceConn: iIRCService loaded");
			iIRCService = i;
			// mSignaliIRCServiceSetup.countDown();
		}
		public void onServiceDisconnected(final ComponentName name) {
			addVerboseLog(4, TAG, "ircServiceConn: onServiceDisconnected()");
			iIRCService = null;
			// layout.setIIRCService(iIRCService);
		}
	};

	/**
	 * IRCサービスのブロードキャストレシーバー登録
	 */
	private void myRegisterReceiver() {
		addVerboseLog(4, TAG, "myRegisterReceiver()");
		final IntentFilter filter = new IntentFilter(IRCService.ACTION);
		registerReceiver(mReceiver, filter);
	}

	/**
	 * IRCサービスからのブロードキャスト受信処理
	 */
	public class IRCServiceReceiver extends BroadcastReceiver {
		@Override
		public synchronized void onReceive(final Context context, final Intent intent) {
			final Bundle extras = intent.getExtras();
			final int msg = extras.getInt("msg");
			switch (msg) {
			case IRCService.RECEIVE_MESSAGE:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: RECEIVE_MESSAGE");
				final String serverName = extras.getString("server");
				final String channel = extras.getString("channel");
				final String nick = extras.getString("nick");
				final String dateMsg = extras.getString("date");
				final SpannableStringBuilder ssb = new SpannableStringBuilder(extras.getCharSequence("ssb"));
				final SpannableStringBuilder ssbOther = new SpannableStringBuilder(extras.getCharSequence("ssbOther"));
				final Boolean alert = extras.getBoolean("alert");

				if (alert) {
					doAlert();
				}
				boolean toMainlog = true;
				boolean toSublog = true;
				if ((mCurrentServerName != null) && (mCurrentChannel != null) &&
						serverName.equalsIgnoreCase(mCurrentServerName) &&
						channel.equalsIgnoreCase(mCurrentChannel)) {
					toSublog = false;
				} else {
					toMainlog = false;
				}
				try {
					if (iIRCService.isPutOnSublog(serverName, channel)) {
						if (iIRCService.isPutOnSublogAll(serverName, channel)) {
							toSublog = true;
						}
					} else {
						toSublog = false;
					}
				} catch (final RemoteException e) {
					e.printStackTrace();
				}
				if (toMainlog) {
					final SpannableStringBuilder ssbLine = IRCMsg.colorDateMsg(dateMsg, SystemConfig.mainLogDateColor[SystemConfig.nowColorSet]);
					if ((nick != null) && (nick.length() > 0)) {
						final SpannableStringBuilder ssbNick = IRCMsg.colorNick(nick, SystemConfig.mainLogTextColor[SystemConfig.nowColorSet]);
						ssbLine.append(ssbNick);
					}
					ssbLine.append(ssb);
					ssbLine.append("\n");
					layout.mMainLogWindow.addMessage(ssbLine);
					try {
						iIRCService.clearChannelUpdated(mCurrentServerName, mCurrentChannel);
					} catch (final RemoteException e) {
						e.printStackTrace();
					}
				}
				if (toSublog) {
					if (SystemConfig.now_subWindowMode > 0) {
						layout.mSubLogWindow.addMessage(ssbOther);
					}
				}
				break;
			}

			case IRCService.CREATE_SERVER:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: CREATE_SERVER");
				final String serverName = extras.getString("server");
				@SuppressWarnings("unused")
				final String dateMsg = extras.getString("date");
				mCurrentServerName = serverName;
				mCurrentServerName = IRCMsg.sSystemChannelName;
				break;
			}

			case IRCService.REMOVE_SERVER:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: REMOVE_SERVER");
				break;
			}

			case IRCService.CREATE_CHANNEL:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: CREATE_CHANNEL");
				final String serverName = extras.getString("server");
				final String channel = extras.getString("channel");
				@SuppressWarnings("unused")
				final String dateMsg = extras.getString("date");
				mCurrentServerName = serverName;
				mCurrentChannel = channel;
				try {
					mTopic = iIRCService.getTopic(serverName, channel);
				} catch (final RemoteException e) {
					e.printStackTrace();
				}
				updateTitle();
				layout.mMainLogWindow.clearLog();
				break;
			}

			case IRCService.REMOVE_CHANNEL:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: REMOVE_CHANNEL");
				break;
			}

			case IRCService.CHANGE_NICK:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: CHANGE_NICK");
				break;
			}

			case IRCService.CHANGE_TOPIC:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: CHANGE_TOPIC");
				final String serverName = extras.getString("server");
				final String channel = extras.getString("channel");
				final String topic = extras.getString("topic");
				@SuppressWarnings("unused")
				final String dateMsg = extras.getString("date");
				if ((mCurrentServerName != null) && (mCurrentChannel != null) &&
						serverName.equalsIgnoreCase(mCurrentServerName) &&
						channel.equalsIgnoreCase(mCurrentChannel)) {
					mTopic = topic;
					updateTitle();
				}
				break;
			}

			case IRCService.PING_PONG:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: PING_PONG");
				try {
					iIRCService.receivePong();
				} catch (final RemoteException e) {
					e.printStackTrace();
				}
				break;
			}

			default:
			{
				addVerboseLog(4, TAG, "IRCServiceReceiver: unknown message: " + msg);
				break;
			}
			}
		}
	}

	/*
	 * オプションメニュー作成
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onCreateOptionsMenu(final Menu menu) {
		addVerboseLog(4, TAG, "onCreateOptionMenu()");
		final MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.menu_main, menu);
		return true;
	}

	/*
	 * オプションメニュー選択処理
	 * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
	public boolean onOptionsItemSelected(final MenuItem item) {
		addVerboseLog(4, TAG, "onOptionsItemSelected()");
		switch (item.getItemId()) {
		case R.id.serverlist:
		{
			doMain.doServerListConfig();
			return true;
		}

		case R.id.help:
		{
			doMain.doShowHelp();
			return true;
		}

		case R.id.version:
		{
			doShowVersion();
			return true;
		}

		case R.id.connect:
		{
			doMain.doConnectAuto();
			return true;
		}

		case R.id.shutdown:
		{
			doMain.doShutdown();
			return true;
		}

		case R.id.setting:
		{
			doMain.doSystemConfig();
			return true;
		}

		case R.id.exapp:
		{
			doMain.doExApp();
			return true;
		}

		case R.id.exapp_edit:
		{
			doMain.doExAppConfig();
			return true;
		}

		case R.id.exweb:
		{
			doMain.doExWeb();
			return true;
		}

		case R.id.exweb_edit:
		{
			doMain.doExWebConfig();
			return true;
		}

		case R.id.chbutton:
		{
			doMain.doChButton();
			return true;
		}

		case R.id.ubutton:
		{
			doMain.doUButton();
			return true;
		}

		case R.id.copybutton:
		{
			doMain.doCopyButton();
			return true;
		}

		case R.id.channel_edit:
		{
			doMain.doChannelConfig();
			return true;
		}

		}
		return false;
	}

	/*
	 * アクティビティの結果処理
	 * @see android.app.Activity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	protected void onActivityResult(final int requestCode, final int resultCode, final Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		addVerboseLog(4, TAG, "onActivityResult()");
		switch (requestCode) {
		case ACTIVITY_SYSTEMCONFIG:
			addVerboseLog(4, TAG, "onActivityResult(): ACTIVITY_SYSTEMCONFIG");
			SystemConfig.loadConfig();
			int rebootLevel = 0;
			if (data != null) {
				final Bundle extras = data.getExtras();
				if (extras != null) {
					rebootLevel = extras.getInt("rebootlevel");
				}
			}
			if ((rebootLevel & 2) != 0) {
				try {
					if (iIRCService != null) {
						iIRCService.reloadSystemConfig();
					}
				} catch (final RemoteException e) {
					e.printStackTrace();
				}
			}
			if ((rebootLevel & 4) != 0) {
				try {
					if (iIRCService != null) {
						iIRCService.clearAllChannelLog();
					}
				} catch (final RemoteException e) {
					e.printStackTrace();
				}
			}
			if ((rebootLevel & 1) != 0) {
				final int fRebootLevel = rebootLevel;
				final ProgressDialog pd = new ProgressDialog(me);
				final Handler h = new Handler();
				final Runnable dismissProgressDialog = new Runnable() {
					public void run() {
						addVerboseLog(4, TAG, "onActivityResult(): ACTIVITY_SYSTEMCONFIG: dismissProgressDialog");
						pd.dismiss();
					}
				};

				final Runnable doRebootLayout = new Runnable() {
					public void run() {
						addVerboseLog(4, TAG, "onActivityResult(): ACTIVITY_SYSTEMCONFIG: doRebootLayout");
						layout.rebootLayout();
						if ((fRebootLevel & 4) != 0) {
							layout.mMainLogWindow.addMessage("Main-log was cleared.", SystemConfig.mainLogTextColor[SystemConfig.nowColorSet], IRCMsg.getDateMsg());
						}
						h.post(dismissProgressDialog);
					}
				};

				final Runnable showProgressDialog = new Runnable() {
					public void run() {
						addVerboseLog(4, TAG, "onActivityResult(): ACTIVITY_SYSTEMCONFIG: showProgressDialog");
						pd.setTitle(getString(R.string.activitymain_java_progress_rebootlayout));
						pd.setIndeterminate(true);
						pd.setCancelable(false);
						pd.show();
						// 少し止めないと変更の反映が速すぎることがある
						/*
							try {
								Thread.sleep(500);
							} catch (final InterruptedException e) {
								//
							}
						 */
						h.post(doRebootLayout);
					}
				};
				h.post(showProgressDialog);
			}

			break;

		case ACTIVITY_EXAPPLIST:
			addVerboseLog(4, TAG, "onActivityResult(): ACTIVITY_EXAPPLIST");
			break;

		case ACTIVITY_EXWEBLIST:
			addVerboseLog(4, TAG, "onActivityResult(): ACTIVITY_EXWEBLIST");
			break;

		case ACTIVITY_IRCCHANNELCONFIG:
			addVerboseLog(4, TAG, "onActivityResult(): ACTIVITY_IRCCHANNELCONFIG");
			try {
				if (iIRCService != null) {
					String currentServerName = iIRCService.getCurrentServerName();
					if (currentServerName == null) break;
					String currentChannel = iIRCService.getCurrentChannel(currentServerName);
					if (currentChannel == null) break;
					iIRCService.reloadChannelConfig(currentServerName, currentChannel);
				}
			} catch (final RemoteException e) {
				e.printStackTrace();
			}
			break;
		}

	}

	/**
	 * Short Toastを表示
	 * @param message メッセージ
	 */
	public static void myShortToastShow(final String message) {
		(me).runOnUiThread(new Runnable(){ public void run() {
			if (mToast != null) {
				mToast.cancel();
			}
			mToast = Toast.makeText(me, message, Toast.LENGTH_SHORT);
			mToast.show();
		} });
	}

	/**
	 * タイトルバーの更新
	 */
	public void updateTitle() {
		if ((mCurrentChannel != null) && (mCurrentServerName != null)) {
			setTitle(IRCMsg.serverChannelTopicTitle(mCurrentServerName, mCurrentChannel, mTopic));
		} else {
			setTitle((mDonate ? "AiCiA (DONATED)" : "AiCiA"));
		}
	}

	/**
	 * アラート処理
	 */
	private void doAlert() {
		// final Vibrator vib = (Vibrator)getSystemService(VIBRATOR_SERVICE);
		// final long[] pat = {200,200,200,200,200,200,200,200};
		// vib.vibrate(pat, -1);
	}

	/**
	 * バージョン表示
	 */
	public void doShowVersion() {
		addVerboseLog(4, TAG, "doShowVersion()");
		String versionName = null;
		final PackageManager pm = getPackageManager();
		try {
			PackageInfo info = null;
			if (ActivityMain.mDonate) {
				info = pm.getPackageInfo("net.gorry.aicia_donate", 0);
			} else {
				info = pm.getPackageInfo("net.gorry.aicia", 0);
			}
			versionName = info.versionName;
		} catch (final NameNotFoundException e) {
			e.printStackTrace();
		}
		final AlertDialog.Builder bldr = new AlertDialog.Builder(me);
		if (ActivityMain.mDonate) {
			bldr.setTitle("AiCiA - Android IRC Client\n(DONATED)");
		} else {
			bldr.setTitle("AiCiA - Android IRC Client");
		}
		bldr.setMessage("Version " + versionName + "\nCopyright (C)2010 GORRY.")
		.setIcon(R.drawable.icon);
		bldr.create().show();
	}

	/**
	 * ログ出力
	 * @param message ログ
	 */
	public static void addVerboseLog(final int level, final String tag, final String message) {
		if (SystemConfig.verbose >= level) {
			if (layout != null) {
				if (layout.mSubLogWindow != null) {
					layout.mSubLogWindow.addMessage(tag + ": " + message, SystemConfig.subLogPaleTextColor[SystemConfig.nowColorSet], IRCMsg.getDateMsg());
				}
			}
		}
		Log.v(tag, message);
	}

}


